<?php

namespace Sukellos;

use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * This class is used to CRUD tracked urls from DB
 * Managed tables :
 * suk_et_tracked_urls
 *
 * @since 1.0.0
 */
class WP_Tracked_Urls_Dao {

    // Use Trait Singleton
    use Singleton;

    /**
     * Insert a new tracked URL
     * @param $url_target
     * @param $token
     * @return int
     */
    public function insert_tracked_url( $url_target, $token, $emails_sent = 0, $name, $description='' ) {

        global $wpdb;
        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        $wpdb->insert(
            $table_name,
            array(
                'url_target' => $url_target,
                'token' => $token,
                'emails_sent' => $emails_sent,
                'created_at' => current_time( 'mysql' ),
                'name' => $name,
                'description' => $description,
            ),
            array( '%s', '%s', '%d', '%s', '%s', '%s' )
        );

        return $wpdb->insert_id;
    }

    /**
     * Update the number of email sent for a tracked URL
     * @param $tracked_url_id
     * @param $emails_sent
     * @return void
     */
    public function update_emails_sent( $tracked_url_id, $emails_sent ) {

        global $wpdb;
        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        $wpdb->update(
            $table_name,
            array( 'emails_sent' => $emails_sent ),
            array( 'id' => $tracked_url_id ),
            array( '%d' ),
            array( '%d' )
        );
    }

    /**
     * Update the description for a tracked URL
     * @param $tracked_url_id
     * @param $description
     * @return void
     */
    public function update_description( $tracked_url_id, $description ) {

        global $wpdb;
        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        $wpdb->update(
            $table_name,
            array( 'description' => $description ),
            array( 'id' => $tracked_url_id ),
            array( '%s' ),
            array( '%d' )
        );
    }


    /**
     * Get querys
     * @return array|object|\stdClass[]|null
     */
    public function get_tracked_urls(  $url_target_filter=null, $token_filter=null, $name_filter=null, $orderby='id', $order='ASC', $offset=0, $limit=50 ) {

        global $wpdb;
        $table_name = $wpdb->prefix."suk_et_tracked_urls";

        $params = array();

        // Build SQL
        $sql = "SELECT * FROM ".$table_name." WHERE 1=1";

        // Add filters
        if ( !is_null( $url_target_filter ) ) {

            $sql .= " AND url_target LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $url_target_filter ) . '%';
        }
        if ( !is_null( $token_filter ) ) {

            $sql .= " AND token LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $token_filter ) . '%';
        }
        if ( !is_null( $name_filter ) ) {

            $sql .= " AND name LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $name_filter ) . '%';
        }

        $sql .= " ORDER BY ".$orderby." ".($order=='desc'?"DESC":"ASC");

        if ( !is_null( $limit ) ) {

            $params[]=$offset;
            $params[]=$limit;
            $sql .= ' LIMIT %d, %d';
        }

        $prepared_statement = $wpdb->prepare($sql, $params);
        $results = $wpdb->get_results( $prepared_statement, 'ARRAY_A' );
        WP_Log::debug(__METHOD__, ['Result of select'=>sizeof($results), 'Statement'=>$prepared_statement], 'wp-sukellos-fw');

        return $results;
    }

    /**
     * Get tracked URL from id
     * @param $token
     * @return array|object|\stdClass|void|null
     */
    public function get_tracked_url( $id ) {

        global $wpdb;

        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        $query = $wpdb->prepare(
            "SELECT * FROM $table_name WHERE id = %d",
            $id
        );

        return $wpdb->get_row( $query, ARRAY_A );
    }

    /**
     * Get tracked URL from token
     * @param $token
     * @return array|object|\stdClass|void|null
     */
    public function get_tracked_url_by_token( $token ) {

        global $wpdb;

        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        $query = $wpdb->prepare(
            "SELECT * FROM $table_name WHERE token = %s",
            $token
        );

        return $wpdb->get_row( $query, ARRAY_A );
    }

    /**
     * Delete a tracked URL
     * @param $token
     * @return bool|int|\mysqli_result|null
     */
    public function delete_tracked_url_by_token( $token ) {

        global $wpdb;

        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        return $wpdb->delete( $table_name, array( 'token' => $token ), array( '%s' ) );
    }

    /**
     * Delete a tracked URL
     * @param $id
     * @return bool|int|\mysqli_result|null
     */
    public function delete_tracked_url( $id ) {

        global $wpdb;

        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        return $wpdb->delete( $table_name, array( 'id' => $id ), array( '%d' ) );
    }

    /**
     * Check if the same cleaned URL does already exist in tracking urls
     * @param $url_target
     * @return bool
     */
    public function is_url_target_exists( $url_target, $name ) {

        global $wpdb;
        $table_name = $wpdb->prefix.'suk_et_tracked_urls';

        // Nettoyer l'URL
        $cleaned_url = Email_Tracker_Url_Manager::clean_url( $url_target );

        // Préparer et exécuter la requête SQL pour vérifier si l'URL existe
        $query = $wpdb->prepare(
            "SELECT COUNT(*) FROM $table_name WHERE (LOWER(TRIM(TRAILING '/' FROM REPLACE(url_target, 'https://', ''))) = %s OR LOWER(TRIM(TRAILING '/' FROM REPLACE(url_target, 'http://', ''))) = %s) AND name = %s",
            $cleaned_url, $cleaned_url, $name
        );

        // Obtenir le résultat
        $count = $wpdb->get_var( $query );
        WP_Log::debug( __METHOD__, ['query'=>$query, 'count'=>$count], 'wp-sukellos-fw');

        return $count > 0;
    }

    /**
     * Get email opens and link clicks for a specific token
     *
     * @param string $token
     * @return array
     */
    public function get_tracking_data_by_token( $token ) {

        global $wpdb;

        // Table names
        $email_opens_table = $wpdb->prefix.'suk_et_email_opens';
        $link_clicks_table = $wpdb->prefix.'suk_et_link_clicks';
        $tracked_urls_table = $wpdb->prefix.'suk_et_tracked_urls';

        // Récupérer l'URL cible associée au token
        $url_data = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $tracked_urls_table WHERE token = %s", $token ) );

        if ( !$url_data ) {
            return array(); // Pas de données trouvées pour ce token
        }

        // Récupérer les ouvertures d'emails pour ce token
        $email_opens = $wpdb->get_results( $wpdb->prepare(
            "SELECT ip_address, user_agent, opened_at FROM $email_opens_table WHERE token = %s",
            $token
        ), ARRAY_A );

        // Récupérer les clics sur les liens pour ce token
        $link_clicks = $wpdb->get_results( $wpdb->prepare(
            "SELECT email, name, ip_address, user_agent, clicked_at FROM $link_clicks_table WHERE token = %s",
            $token
        ), ARRAY_A );

        return array(
            'url_target' => $url_data->url_target,
            'emails_sent' => $url_data->emails_sent,
            'email_opens' => $email_opens,
            'link_clicks' => $link_clicks
        );
    }

}