<?php

namespace Sukellos;

use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * Email tracker URL management class.
 *
 * @since 1.0.0
 */
class Email_Tracker_Url_Manager {

    // Use Trait Singleton
    use Singleton;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

    }

    /**
     * Clean an URL
     * @param $url
     * @return string
     */
    public static function clean_url( $url ) {

        // Supprimer "http://" ou "https://"
        $url = preg_replace( '#^https?://#', '', rtrim( $url, '/' ) );

        // Convertir en minuscules
        $url = strtolower( $url );

        return $url;
    }

    /**
     * Generate a new fere unique ATI3D key
     * @return false|string
     */
    private function generate_unique_token() {

        $is_unique_token = false;
        $unique_token = null;
        while ( !$is_unique_token ) {

            // Generate key
            $unique_token = "";
            $chaine = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
            srand( (double)microtime() * 1000000 );
            for ( $i = 0; $i < 25; $i++ ) {
                $unique_token .= $chaine[ rand() % strlen( $chaine ) ];
            }

            // Try and get a licence with these key from BDD
            $result = WP_Tracked_Urls_Dao::instance()->get_tracked_url_by_token( $unique_token );
            if ( is_null( $result ) ) $is_unique_token = true;
        }

        return $unique_token;
    }

    /**
     * Delete tracked url and all related content
     * @param $cle
     * @return false|void
     */
    public function delete_tracked_url( $tracked_url_id ) {

        WP_Log::debug( __METHOD__, ['$tracked_url_id'=>$tracked_url_id], 'wp-sukellos-fw' );

        if ( is_null( $tracked_url_id ) ) {

            WP_Log::debug( __METHOD__.' - Missing tracked URL id, nothing deleted', [], 'wp-sukellos-fw' );

            return false;
        }

        WP_Tracked_Urls_Dao::instance()->delete_tracked_url( $tracked_url_id );
        WP_Email_Opens_Dao::instance()->delete_email_opens_by_tracked_url( $tracked_url_id );
        WP_Link_Clicks_Dao::instance()->delete_link_clicks_by_tracked_url( $tracked_url_id );
    }

    /**
     * Generate a new tranckingURL from a tracked URL
     * @param $tracked_url
     * @param $nb_emails_sent
     * @return void
     */
    public function generate_tracking_url( $tracked_url, $nb_emails_sent=0, $name, $description='' ) {

        if ( ( ( is_null( $tracked_url ) || empty( $tracked_url ) ) ) || ( ( is_null( $name ) || empty( $name ) ) ) ) {

            WP_Log::debug( __METHOD__.' - Missing info, tracking url not generated', ['$tracked_url'=>$tracked_url, 'name'=>$name, '$nb_emails_sent'=>$nb_emails_sent], 'wp-sukellos-fw' );

            return false;
        }
        // Check if the URL is already tracked, then not generated
        $is_url_target_exists = WP_Tracked_Urls_Dao::instance()->is_url_target_exists( $tracked_url, $name );
        if ( $is_url_target_exists ) {

            WP_Log::debug( __METHOD__.' - Target urls already tracked', ['$tracked_url'=>$tracked_url], 'wp-sukellos-fw' );

            return false;
        }

        // Generate the token
        $token = $this->generate_unique_token();

        // Insert in DB
        WP_Tracked_Urls_Dao::instance()->insert_tracked_url( $tracked_url, $token, $nb_emails_sent, $name, $description );
    }

    /**
     * Record an avent : link_click
     * @param $token
     * @return void
     */
    public function record_link_click( $token, $email, $name, $ip_address, $user_agent ) {

        // Get url tracking id
        $tracked_url = WP_Tracked_Urls_Dao::instance()->get_tracked_url_by_token( $token );

        if ( is_null( $tracked_url ) ) {

            WP_Log::debug( __METHOD__.' - Token does not exist in tracking urls', [ '$token' => $token ], 'wp-sukellos-fw' );

            return false;
        }

        // Insert an entry email_open
        WP_Link_Clicks_Dao::instance()->insert_link_click( $tracked_url[ 'id' ], $token, $email, $name, $ip_address, $user_agent );
        return $tracked_url['url_target'];
    }

    /**
     * Record an avent : email open
     * @param $token
     * @return void
     */
    public function record_email_open( $token, $ip_address, $user_agent ) {

        // Get url tracking id
        $tracked_url = WP_Tracked_Urls_Dao::instance()->get_tracked_url_by_token( $token );

        if ( is_null( $tracked_url ) ) {

            WP_Log::debug( __METHOD__.' - Token does not exist in tracking urls', [ '$token' => $token ], 'wp-sukellos-fw' );

            return false;
        }

        // Insert an entry email_open
        WP_Email_Opens_Dao::instance()->insert_email_open( $tracked_url[ 'id' ], $token, $ip_address, $user_agent );
        return $tracked_url['url_target'];
    }

    /**
     * Build the img HTML to insert in mail from the token
     * @param $token
     * @return string
     */
    public function get_email_open_tracking_image( $token ) {

        // Build URL to track email open
        $tracking_url = get_site_url().Email_Tracking_Webservices::URL_SUFFIX_EMAIL_OPEN.'?token='.urlencode( $token );

        // Générer la balise <img> avec l'URL de suivi
        $img_tag = '<img src="'.esc_url( $tracking_url ).'" width="1" height="1" style="display:none;" alt=""/>';

        return $img_tag;
    }

    /**
     * Build the URL link to substitute in mail from the token
     * @param $token
     * @return string
     */
    public function get_link_click_tracking_url( $token, $with_examples=false ) {

        // Build URL to track link_click
        $tracking_url = get_site_url().Email_Tracking_Webservices::URL_SUFFIX_LINK_CLICK.'?token='.urlencode( $token );

        if ( $with_examples ) {

            $tracking_url .= '&email=${MyEmailTemplate.E-mail}&name=${MyEmailTemplate.Name}';
        }

        return $tracking_url;
    }

    /**
     * Groupe opens and clicks per email and user agent
     *
     * @param array $data
     * @return array
     */
    public function get_tracking_data_by_token( $token ) {

        $data = WP_Tracked_Urls_Dao::instance()->get_tracking_data_by_token( $token );
        if ( is_null( $data ) ) return array( null, null, null );

        $grouped_data = [];

        // Fusionner les ouvertures d'emails et les clics
        $all_entries = array_merge( $data[ 'email_opens' ], $data[ 'link_clicks' ] );

        foreach ( $all_entries as $entry ) {

            $key = $entry[ 'ip_address' ].'-'.$entry[ 'user_agent' ];

            if ( !isset( $grouped_data[ $key ] ) ) {

                $grouped_data[ $key ] = [
                    'email' => $entry[ 'email' ] ?? '-',
                    'name' => $entry[ 'name' ] ?? '-',
                    'ip_address' => $entry[ 'ip_address' ],
                    'user_agent' => $entry[ 'user_agent' ],
                    'emails_opened' => isset( $entry[ 'opened_at' ] ) ? 1 : 0,
                    'links_clicked' => isset( $entry[ 'clicked_at' ] ) ? 1 : 0
                ];
            } else {

                // Augmenter le nombre d'ouvertures ou de clics si déjà existant
                if ( isset( $entry[ 'opened_at' ] ) ) {

                    $grouped_data[ $key ][ 'emails_opened' ]++;
                }
                if ( isset( $entry[ 'clicked_at' ] ) ) {

                    $grouped_data[ $key ][ 'links_clicked' ]++;
                }
            }
        }

        return array( $data[ 'url_target' ], $data[ 'emails_sent' ], $grouped_data );
    }

}
