<?php
/**
 * Web service : tracking requests
 *
 * @author      Sukellos
 * @copyright   Copyright (c) 2021-2024, Sukellos.
 * @license     Private
 */

namespace Sukellos;

use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;
use WP_REST_Request;
use WP_REST_Response;
use Exception;

defined( 'ABSPATH' ) or exit;

/**
 * WP Admin menu management class.
 *
 * @since 1.0.0
 */
class Email_Tracking_Webservices {

    // Use Trait Singleton
    use Singleton;

    const URL_SUFFIX_EMAIL_OPEN = '/wp-json/suket/email_open/';
    const URL_SUFFIX_LINK_CLICK = '/wp-json/suket/link_click/';

    const PARAM_EMAIL = 'email';
    const PARAM_NAME = 'name';

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        // Fires when preparing to serve a REST API request.
        // See https://developer.wordpress.org/reference/hooks/rest_api_init/
        add_action('rest_api_init', array( $this, 'action_rest_api_init' ), 10, 1 );
    }

    /**
     * Preparing to serve a REST API request.
     *
     * @param $wp_rest_server
     * @return void
     */
    public function action_rest_api_init( $wp_rest_server ) {

        // email_open tracking
        // /wp-json/suket/email_open/?token=BTGN2JM970P2LW70C2JFY0CR6
        register_rest_route('suket', '/email_open/', array(
            'methods' => 'GET',
            'callback' => array( $this, 'process_email_open_request' ),
        ));

        // link_click tracking
        // /wp-json/suket/link_click/?token=BTGN2JM970P2LW70C2JFY0CR6&email=<email>&name=<name>
        register_rest_route('suket', '/link_click/', array(
            'methods' => 'GET',
            'callback' => array( $this, 'process_link_click_request' ),
        ));

    }

    /**
     * Process the email_open request
     * <img src="https://yourserver.com/wp-json/suket/email_open/?token=BTGN2JM970P2LW70C2JFY0CR6" width="1" height="1" alt="">
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response
     */
    public function process_email_open_request(WP_REST_Request $request) {

        WP_Log::debug( __METHOD__, ['request'=>$request], 'wp-sukellos-fw' );

        // Get request parameters
        $token = $request->get_param( 'token' );

        // Check if token given
        if ( empty( $token ) ) {

            return new WP_REST_Response( 'Token is missing', 400 );
        }

        // Get IP and User-Agent
        $ip_address = $request->get_header( 'X-Forwarded-For' ) ?: $request->get_header( 'REMOTE_ADDR' );
        $user_agent = $request->get_header( 'user-agent' );
        WP_Log::debug( __METHOD__, ['$ip_address'=>$ip_address, '$user_agent'=>$user_agent], 'wp-sukellos-fw' );

        // REcord event
        $url_target = Email_Tracker_Url_Manager::instance()->record_email_open( $token, $ip_address, $user_agent );

        if ( !$url_target ) {

            return new WP_REST_Response( 'Token not found', 404 );
        }

        // Retourner une image vide 1x1
        $image_data = base64_decode(
            'R0lGODlhAQABAIABAP///wAAACH5BAEKAAAALAAAAAABAAEAAAICRAEAOw=='
        ); // Image GIF 1x1 encodée en base64

        $response = new WP_REST_Response($image_data);
        $response->header('Content-Type', 'image/gif');

        return $response;
    }

    /**
     * Process the link_click request
     *
     * @param WP_REST_Request $request
     * @return WP_REST_Response
     */
    public function process_link_click_request( WP_REST_Request $request ) {

        WP_Log::debug( __METHOD__, ['request'=>$request], 'wp-sukellos-fw' );

        // Get request parameters
        $token = $request->get_param( 'token' );
        $email = $request->get_param( self::PARAM_EMAIL ); // Facultatif
        $name = $request->get_param( self::PARAM_NAME );   // Facultatif

        // Check if token given
        if ( empty( $token ) ) {

            return new WP_REST_Response( 'Token is missing', 400 );
        }

        // Get IP and User-Agent
        $ip_address = $request->get_header('X-Forwarded-For') ? explode(',', $request->get_header('X-Forwarded-For'))[0] : $request->get_header('REMOTE_ADDR');
        $ip_address = !empty($_SERVER['HTTP_X_FORWARDED_FOR']) ? explode(',', $_SERVER['HTTP_X_FORWARDED_FOR'])[0] : $_SERVER['REMOTE_ADDR'];

        $user_agent = $request->get_header( 'user-agent' );
        WP_Log::debug( __METHOD__, ['$ip_address'=>$ip_address, '$user_agent'=>$user_agent], 'wp-sukellos-fw' );

        // Record event
        $url_target = Email_Tracker_Url_Manager::instance()->record_link_click( $token, $email, $name, $ip_address, $user_agent );

        if ( !$url_target ) {

            return new WP_REST_Response( 'Token not found', 404 );
        }

        // Redirection 301 permanente
        return new WP_REST_Response(
            array(
                'message' => 'Redirecting...',
                'url' => $url_target
            ),
            301,
            array( 'Location' => $url_target )
        );
    }
}
