<?php
/**
 * Plugin Name: Sukellos Email Tracker
 * Plugin URI: https://wp-adminbuilder.com/email-tracker/
 * Description: Email Tracker on Wordpress, by Sukellos
 * Version: 1.0.2
 * Requires at least: 6.6.2
 * Requires PHP:      8.0
 * Author: Sukellos
 * Author URI: https://sukellos.com
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: wp-sukellos-fw
 * Domain Path: /languages
 *
 * Copyright: (c) 2021-2024 Sukellos, SARL (ludovic.maillet@sukellos.fr)
 *
 * @package   WP-Sukellos-Email-Tracker
 * @author    Sukellos
 * @category  Admin
 * @copyright Copyright (c) 2021-2024 Sukellos, SARL
 * @license   Private
 */
namespace Sukellos;

defined( 'ABSPATH' ) or exit;

// Sukellos plugins share the same inclusion to allow regrouping whole Tools into one centralized admin page
// List authorized basenames for inclusion for security purpose
$authorized_sukellos_plugins = array(
    'sukellos-admin-builder',
    'sukellos-admin-builder-basic',
    'sukellos-dashboard-bar',
    'sukellos-email-tracker',
    'sukellos-enable-classic-editor',
    'sukellos-image-formats',
    'sukellos-logger',
    'sukellos-login-style',
    'sukellos-login-wrapper',
    'sukellos-leo2-wooc',
    'sukellos-scrapio',
);
// Resolve Sukellos Fw inclusion
$wp_sukellos_fw_dir_ref = wp_cache_get( 'wp_sukellos_fw_dir_ref', 'wp_sukellos_fw' );
$authorized = false;
if ( FALSE !== $wp_sukellos_fw_dir_ref ) {

    // Ref must be one of authorized paths
    foreach ( $authorized_sukellos_plugins as $authorized_sukellos_plugin ) {

        $haystack_length = strlen( $wp_sukellos_fw_dir_ref );
        $needle_length = strlen( $authorized_sukellos_plugin );
        // Sukellos needle must be at the end of the haystack
        $needle_pos = strpos( $wp_sukellos_fw_dir_ref, $authorized_sukellos_plugin );
        $awaited_pos = $haystack_length - $needle_length;
        if ( ( FALSE !== $needle_pos ) && ( $needle_pos === $awaited_pos ) ) {

            $authorized = true;
        }
    }
}

// Ref was not present, or not authorized, then force current plugin ref on itself
if ( ( FALSE === $wp_sukellos_fw_dir_ref ) || ( !$authorized ) ) {

    // This one becomes the ref
    $wp_sukellos_fw_dir_ref = __DIR__;
    wp_cache_set( 'wp_sukellos_fw_dir_ref', $wp_sukellos_fw_dir_ref, 'wp_sukellos_fw' );
}

// Require vendor autoloads to be able to Use all frameworks namespaces
require_once $wp_sukellos_fw_dir_ref . '/vendor/autoload.php';
require_once $wp_sukellos_fw_dir_ref . '/vendor/wp-sukellos-fw/autoload.php';

// Require autoload for this current plugin
require_once __DIR__ . '/autoload.php';

// Sukellos Framework
use Sukellos\WPFw\Singleton;
use Sukellos\Admin\WP_Sukellos_Email_Tracker_Admin;
use Sukellos\WPFw\WP_Sukellos_Fw_Loader;


/**
 * The loader class.
 *
 * @since 1.0.0
 */
final class WP_Sukellos_Email_Tracker_Loader extends WP_Sukellos_Fw_Loader {

    // Use Trait Singleton
    use Singleton;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        load_plugin_textdomain( 'wp-sukellos-fw', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

        parent::init();
    }

    /**
     * Must be called in child Loader to get data from the file itself
     */
    public function get_plugin_file() {

        return __FILE__;
    }

    /**
     * Get the plugin instance
     *
     * @since 1.0.0
     *
     * @return The plugin main instance
     */
    public function get_plugin() {

        return WP_Sukellos_Email_Tracker::instance();
    }

    /**
     * Gets the admin instance.
     *
     * @since 1.0.0
     *
     * @return admin instance, or null is no admin enabled
     */
    public function get_plugin_admin() {

        return WP_Sukellos_Email_Tracker_Admin::instance();
    }

    /**
     * Activate plugin can be used to create data, create tables...
     *
     * @since 1.0.0
     */
    public function activate() {

        // MANDATORY to call it before anything else
        parent::activate();

        // Create table for MySQL Monolog logging

        // You must put each field on its own line in your SQL statement.
        // You must have two spaces between the words PRIMARY KEY and the definition of your primary key.
        // You must use the key word KEY rather than its synonym INDEX and you must include at least one KEY.
        // You must not use any apostrophes or backticks around field names.
        // Field types must be all lowercase.
        // SQL keywords, like CREATE TABLE and UPDATE, must be uppercase.
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();


        // You must put each field on its own line in your SQL statement.
        // You must have two spaces between the words PRIMARY KEY and the definition of your primary key.
        // You must use the key word KEY rather than its synonym INDEX and you must include at least one KEY.
        // You must not use any apostrophes or backticks around field names.
        // Field types must be all lowercase.
        // SQL keywords, like CREATE TABLE and UPDATE, must be uppercase.

        // Stats clicks on links
        $table_name = $wpdb->prefix."suk_et_tracked_urls";

        $sql = "CREATE TABLE ".$table_name." (
              id INT AUTO_INCREMENT PRIMARY KEY,
              url_target VARCHAR(255) NOT NULL,
              token VARCHAR(255) UNIQUE NOT NULL,
              emails_sent BIGINT UNSIGNED DEFAULT 0, 
              created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
              name VARCHAR(255) NOT NULL,
              description TEXT,
              INDEX idx_token (token) 
        ) ".$charset_collate.";";

        require_once( ABSPATH.'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );

        // Stats clicks on links
        $table_name = $wpdb->prefix."suk_et_email_opens";

        $sql = "CREATE TABLE ".$table_name." (
              id INT AUTO_INCREMENT PRIMARY KEY,
              tracked_url_id INT,
              token VARCHAR(255),
              opened_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
              ip_address VARCHAR(45),
              user_agent TEXT,
              INDEX idx_token (token) 
        ) ".$charset_collate.";";

        require_once( ABSPATH.'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );

        // Stats clicks on links
        $table_name = $wpdb->prefix."suk_et_link_clicks";

        $sql = "CREATE TABLE ".$table_name." (
              id INT AUTO_INCREMENT PRIMARY KEY,
              tracked_url_id INT,
              token VARCHAR(255),
              email VARCHAR(255),
              name VARCHAR(255),
              clicked_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP, 
              ip_address VARCHAR(45),
              user_agent TEXT,
              INDEX idx_token (token) 
        ) ".$charset_collate.";";

        require_once( ABSPATH.'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    }
}
WP_Sukellos_Email_Tracker_Loader::instance();