<?php

namespace Sukellos\WPFw\AdminBuilder;

use Sukellos\WPFw\Utils\WP_Admin_Notices_Manager;
use Sukellos\WPFw\Utils\WP_Helper;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * Item_Form inherit from Container Class
 *
 * It is contained in a page, and just check mandatory and forward to "human" handler
 *
 * Allow treatment / adding a new setting on any Item field : mandatory => mandatory text
 *
 * @since 1.3.0
 */
class Admin_Item_Form extends Admin_Container {

    private $default_item_group_settings = array(

        /**
         * Used to know if an item should be displayed independently of a global form.
         * This option is used to break a form rendering when the item is displayed.
         * It can be used in conjunction with need_form to force own form for a field
         * Or to exclude a content item from a global form
         * Default for item group: true
         *
         * @since 1.0.0
         */
        'global_form_independent' => true,

        /**
         * Used to display a custom success message
         * Default: null for default message
         *
         * @since 1.3.3
         */
        'success_message' => null,
    );

    private $message = null;

    const TYPE_SEPARATOR = '!!';

    /**
     * Constructor
     *
     * @param $settings
     */
    public function __construct( $settings ) {

        parent::__construct( $settings );

        // Set type
        $this->set_type( Item_Type::ADMIN_ITEM_FORM );

        if ( is_null( $this->get_id() ) || ( $this->get_id() === '' ) ) {

            $generated_id = str_replace( ' ', '-', trim( strtolower( $this->get_name() ) ) );
            $this->set_id( $generated_id );
        }

        /**
         * Action : sukellos_fw/admin_builder/admin_page/fields_loaded
         * Called after the fields ar loaded
         */
        add_action( 'sukellos_fw/admin_builder/admin_page/fields_loaded', array( $this, 'action_fields_loaded' )  );

        /**
         * Action : sukellos_fw/admin_builder/admin_page/fields_saved
         * Called after the fields are saved
         * Used to verify mandatories
         */
        add_action( 'sukellos_fw/admin_builder/admin_page/fields_saved', array( $this, 'action_fields_saved' ) );

        add_filter( 'sukellos_fw/admin_builder/admin_page/get_message', array( $this, 'filter_get_message' ), 10, 1 );

        WP_Log::debug(__METHOD__, ['id'=>$this->get_id()]);
    }

    /**
     * Get the default item settings
     * @return array
     */
    protected function get_default_specific_admin_container_settings() {

        return $this->default_item_group_settings;
    }

    /**
     * Verify a few things before saving fields
     * @return mixed
     */
    public function verify_security() {

        return true;
    }

    /**
     * Change action save fields message
     */
    public function filter_get_message( $message ) {

        WP_Log::debug( __METHOD__, ['GET'=>$_GET, 'message'=>$message, '$this->message'=>$this->message, 'parent_id'=>$this->get_parent_id()] );

        // Is the container concerned ?
        if ( empty( $_GET ) || !isset( $_GET['page'] ) || ( $_GET['page'] != $this->get_parent_id() ) ) {

            WP_Log::debug(__METHOD__.' - Not concerned by this hook', ['GET'=>$_GET, 'message'=>$message, '$this->message'=>$this->message, 'parent_id'=>$this->get_parent_id()] );
            return $message;
        }

        if ( !is_null( $this->message ) ) {

            return $this->message;
        }
        return $message;
    }

    /**
     * Check if mandatories are submitted
     * @return void
     */
    public function action_fields_saved() {

        WP_Log::debug( __METHOD__, ['GET'=>$_GET, 'POST'=>$_POST, 'parent_id'=>$this->get_parent_id()] );

        // Check if it is concerned page
        if ( isset( $_GET['page'] ) && ( $_GET['page'] !== $this->get_parent_id() ) ) {

            WP_Log::debug( __METHOD__.' Not concerned', ['GET'=>$_GET, 'POST'=>$_POST, 'parent_id'=>$this->get_parent_id()] );
            return;
        }

        // Check all items mandatory
        $items = $this->get_items();
        $valid = true;

        $this->message = WP_Helper::sk__('Form successfully saved' ).self::TYPE_SEPARATOR.WP_Admin_Notices_Manager::TYPE_SAVED;
        if ( !is_null( $this->get_success_message() ) ) {
            $this->message = $this->get_success_message().self::TYPE_SEPARATOR.WP_Admin_Notices_Manager::TYPE_SAVED;
        }
        foreach ( $items as $item ) {

            WP_Log::debug( __METHOD__, ['item settings'=>$item->get_settings()] );
            $mandatory = $item->get_mandatory();
            WP_Log::debug( __METHOD__, ['mandatory'=>is_null($mandatory)?'null':(($mandatory===false)?'false':(empty($mandatory)?'empty':$mandatory))] );

            if ( !empty($mandatory) ) {

                if ( !isset( $_POST[ ''.$item->get_id() ] )  || ( empty( $_POST[ ''.$item->get_id() ] )  ) ) {

                    $this->message = $mandatory.self::TYPE_SEPARATOR.WP_Admin_Notices_Manager::TYPE_ERROR;
                    WP_Log::debug(__METHOD__.' - Form mandatory field empty or not submitted', ['POST'=>$_POST, 'message'=>$this->message] );

                    $valid = false;

                    break;
                }
            }
        }

        /**
         * Action : sukellos_fw/admin_builder/item_form/fields_submitted_{form_item_id}
         * Called after the fields are submitted and mandatories verified in form item
         */
        if ( $valid ) do_action( 'sukellos_fw/admin_builder/item_form/fields_submitted_'.$this->get_id() );

    }

    /**
     * Create an option
     *
     * @param array $settings depending on option
     */
    public function create_form_field( $settings ) {

        WP_Log::debug( __METHOD__, ['$settings'=>$settings] );
        $field = $this->create_field( Item_Factory::FORM_FIELD, $settings );

        // No Hook saving
        add_action( 'sukellos_fw/admin_builder/admin_page/save_field', array( $field, 'save' ), 5 );
        add_action( 'sukellos_fw/admin_builder/admin_page/load_field', array( $field, 'load' ), 5 );
        add_action( 'sukellos_fw/admin_builder/admin_page/reset_field', array( $field, 'reset' ), 5 );

        return $field;
    }

    /**
     * Wordpress admin page creation (registered in admin_menu)
     *
     * @param bool $echo Whether to display or return string, default true
     */
    public function render( $echo=true ) {

        /*$html_content = '
            <div class="sk-admin-builder-grid-form">
                <div class="sk-admin-builder-group-name">'.esc_html( $this->get_name() ).'</div>
                <div class="sk-admin-builder-group-content">
            ';*/

        $html_content = $this->render_items( $this->get_parent_id(), false );

        /*$html_content .= '
                </div>
            </div>
            ';*/

        if ( $echo ) {

            echo $html_content;
        } else {

            return $html_content;
        }
    }

    /**
     *          ===============
     *      =======================
     *  ============ HOOKS ===========
     *      =======================
     *          ===============
     */


    /**
     * Enqueue the scripts
     *
     * @since 1.0.0
     *
     * @return void
     */
    public function enqueue_scripts() {}

    /**
     * Load the javascript
     *
     * @since 1.0.0
     *
     * @return void
     */
    public function init_scripts() {}
}
